@@ -33,6 +33,8 @@ GNOME_CLASS_BOILERPLATE (SplashWidget,
 			 GObject,
 			 GTK_TYPE_WINDOW);
 
+static gboolean update_trans_effect (gpointer);
+
 typedef struct {
 	char *human_name;
 	char *exe;
@@ -40,13 +42,12 @@ typedef struct {
 } SplashApp;
 
 static const SplashApp splash_map_table[] = {
-	{ N_("Sawfish Window Manager"),  "sawfish",               "gnome-ccwindowmanager.png" },
-	{ N_("Metacity Window Manager"), "metacity",              "gnome-ccwindowmanager.png" },
-	{ N_("Window Manager"),          "gnome-wm",              "gnome-ccwindowmanager.png" },
-	{ N_("The Panel"),               "gnome-panel",           "gnome-panel.png" },
-	{ N_("Session Manager Proxy"),   "gnome-smproxy",         "gnome-session.png" },
-	{ N_("Nautilus"),                "nautilus",              "gnome-ccdesktop.png" },
-	{ N_("Desktop Settings"),        "gnome-settings-daemon", "gnome-settings.png" }
+	{ N_("Sawfish Window Manager"),  "sawfish",               "gnome-window-manager" },
+	{ N_("Metacity Window Manager"), "metacity",              "gnome-window-manager" },
+	{ N_("Window Manager"),          "gnome-wm",              "gnome-window-manager" },
+	{ N_("The Panel"),               "gnome-panel",           "gnome-panel" },
+	{ N_("Nautilus"),                "nautilus",              "gnome-fs-desktop" },
+	{ N_("Desktop Settings"),        "gnome-settings-daemon", "gnome-settings" }
 };
 
 static const SplashApp *
@@ -66,8 +67,22 @@ typedef struct {
 	GdkRectangle position;
 	GdkPixbuf   *unscaled;
 	GdkPixbuf   *scaled;
+	GdkPixbuf   *scaled_copy;
+	gint	     trans_count;
 } SplashIcon;
 
+typedef struct {
+	SplashWidget *sw;
+	SplashIcon   *si;
+	int width;
+	int height;
+	int n_channels;
+	int rowstride_trans;
+	int rowstride_orig;
+	guchar *pixels_trans;
+	guchar *pixels_orig;
+} TransParam;
+
 static gboolean
 re_scale (SplashWidget *sw)
 {
@@ -119,8 +134,8 @@ calc_text_box (SplashWidget *sw)
 			  pixel_rect.width / 2);
 	sw->text_box.y = (sw->image_bounds.y + sw->image_bounds.height -
 			  pixel_rect.height - SPLASH_LABEL_V_OFFSET);
-	sw->text_box.width = pixel_rect.width;
-	sw->text_box.height = pixel_rect.height;
+	sw->text_box.width = pixel_rect.width + 1;
+	sw->text_box.height = pixel_rect.height + 1;
 }
 
 static gboolean
@@ -268,7 +283,9 @@ splash_icon_destroy (SplashIcon *si)
 	g_object_unref (si->unscaled);
 	if (si->scaled)
 		g_object_unref (si->scaled);
-
+	if (si->scaled_copy)
+		g_object_unref (si->scaled_copy);
+	
 	g_free (si);
 }
 
@@ -283,7 +300,12 @@ splash_widget_finalize (GObject *object)
 	g_list_foreach (sw->icons, (GFunc) splash_icon_destroy, NULL);
 	g_list_free (sw->icons);
 
+	if (sw->background)
+		g_object_unref (sw->background);
+	sw->background = NULL;
+
 	g_object_unref (sw->layout);
+	sw->layout = NULL;
 
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
@@ -381,8 +403,10 @@ splash_widget_instance_init (SplashWidget *sw)
 	pango_layout_set_alignment (sw->layout, PANGO_ALIGN_CENTER);
 
 	attrs = pango_attr_list_new ();
-	pango_attr_list_insert (attrs,
-				pango_attr_size_new (SPLASH_LABEL_FONT_SIZE));
+	sw->font_size_attr = pango_attr_size_new (PANGO_SCALE * SPLASH_LABEL_FONT_SIZE);
+	sw->font_size_attr->start_index = 0;
+	sw->font_size_attr->end_index = 0;
+	pango_attr_list_insert (attrs, sw->font_size_attr);
 	pango_layout_set_attributes (sw->layout, attrs);
 	pango_attr_list_unref (attrs);
 
@@ -445,6 +469,9 @@ layout_icon (SplashWidget *sw, SplashIcon *si, GdkRectangle *area)
 			  (sw->icon_size + sw->icon_spacing) * sw->cur_y_row);
 	si->position.width = si->position.height = sw->icon_size;
 
+	if (gtk_widget_get_direction (GTK_WIDGET (sw)) == GTK_TEXT_DIR_RTL)
+		si->position.x = GTK_WIDGET (sw)->allocation.width - si->position.x - si->position.width;
+
 	sw->cur_x_offset += sw->icon_size + sw->icon_spacing;
 
 	if (area)
@@ -492,64 +519,59 @@ re_laydown (SplashWidget *sw)
 		layout_icon (sw, l->data, NULL);
 	}
 }
- 
+
 void
 splash_widget_add_icon (SplashWidget *sw,
 			const char   *executable_name)
 {
-	char *text;
-	char *basename;
-	GdkPixbuf *pb;
 	const SplashApp *app;
+	char            *basename;
+	GdkPixbuf       *pb = NULL;
 
 	g_return_if_fail (SPLASH_IS_WIDGET (sw));
 
 	if (!executable_name || executable_name [0] == '\0')
 		return;
 
-	basename = g_path_get_basename (executable_name);
-
-	pb = NULL;
-	text = NULL;
-	app = get_splash_app (basename);
-
-	if (app) {
-		pb = get_splash_icon (sw, app->icon);
-		text = _(app->human_name);
-	}
-
 	/* FIXME: we should allow arbitrary apps to install
          * stuff here, by passing the icon name to the SM
          * as a session property
          */
 
+	basename = g_path_get_basename (executable_name);
+	app = get_splash_app (basename);
+
+	if (app)
+		pb = get_splash_icon (sw, app->icon);
+
 	if (!pb)
 		pb = get_splash_icon (sw, basename);
 
-	if (!pb)
-		pb = get_splash_icon (sw, "gnome-unknown.png");
-
-	if (!text)
-		text = basename;
-
-	/* re-draw the old text extents */
-	gtk_widget_queue_draw_area (
-		GTK_WIDGET (sw),
-		sw->text_box.x, sw->text_box.y,
-		sw->text_box.width, sw->text_box.height);
-
-	pango_layout_set_text (sw->layout, text, -1);
-	calc_text_box (sw);
-
-	/* re-draw the new text extents */
-	gtk_widget_queue_draw_area (
-		GTK_WIDGET (sw),
-		sw->text_box.x, sw->text_box.y,
-		sw->text_box.width, sw->text_box.height);
-
 	if (pb) {
-		SplashIcon *si;
-		GdkRectangle area;
+		SplashIcon   *si;
+		GdkRectangle  area;
+		char         *text;
+		int           length;
+		TransParam   *tp;
+
+		text = app && app->human_name ?  _(app->human_name) : basename;
+
+		/* re-draw the old text extents */
+		gtk_widget_queue_draw_area (
+			GTK_WIDGET (sw),
+			sw->text_box.x, sw->text_box.y,
+			sw->text_box.width, sw->text_box.height);
+
+		length = strlen (text);
+		sw->font_size_attr->end_index = length;
+		pango_layout_set_text (sw->layout, text, length);
+		calc_text_box (sw);
+
+		/* re-draw the new text extents */
+		gtk_widget_queue_draw_area (
+			GTK_WIDGET (sw),
+			sw->text_box.x, sw->text_box.y,
+			sw->text_box.width, sw->text_box.height);
 
 		si = g_new0 (SplashIcon, 1);
 		si->unscaled = pb;
@@ -557,16 +579,91 @@ splash_widget_add_icon (SplashWidget *sw,
 		sw->icons = g_list_append (sw->icons, si);
 
 		layout_icon (sw, si, &area);
-		
-		gtk_widget_queue_draw_area (
-			GTK_WIDGET (sw),
-			area.x, area.y,
-			area.width, area.height);
+	
+		/* prepare transparency effect */
+		if ((gdk_pixbuf_get_colorspace (si->scaled) == GDK_COLORSPACE_RGB) &&
+		    (gdk_pixbuf_get_bits_per_sample (si->scaled) == 8) &&
+		    (gdk_pixbuf_get_n_channels (si->scaled))) {
+			si->trans_count = 0;
+			if (!gdk_pixbuf_get_has_alpha (si->scaled)) {
+				si->scaled_copy = gdk_pixbuf_add_alpha (si->scaled, FALSE, 0, 0, 0);
+				g_object_unref (si->scaled);
+				si->scaled = gdk_pixbuf_copy (si->scaled_copy);
+			} else 
+				si->scaled_copy = gdk_pixbuf_copy (si->scaled);
+
+			tp = g_new0(TransParam, 1);
+			tp->si = si;
+			tp->sw = sw;
+			tp->width  = gdk_pixbuf_get_width  (tp->si->scaled);
+			tp->height = gdk_pixbuf_get_height (tp->si->scaled);
+			tp->rowstride_trans = gdk_pixbuf_get_rowstride (tp->si->scaled);
+			tp->rowstride_orig  = gdk_pixbuf_get_rowstride (tp->si->scaled_copy);
+			tp->pixels_trans = gdk_pixbuf_get_pixels (tp->si->scaled);
+			tp->pixels_orig  = gdk_pixbuf_get_pixels (tp->si->scaled_copy);
+			tp->n_channels = gdk_pixbuf_get_n_channels (tp->si->scaled);
+
+			gdk_pixbuf_fill (si->scaled, 0x00000000);
+			g_timeout_add (TRANS_TIMEOUT, update_trans_effect, tp);
+		} else {
+			gtk_widget_queue_draw_area (GTK_WIDGET (sw), area.x, area.y,
+						    area.width, area.height);
+		}
 	}
+
+	g_free (basename);
 }
 
 static SplashWidget *global_splash = NULL;
 
+static gboolean
+update_trans_effect (gpointer trans_param)
+{
+	guchar     *p_trans;
+	guchar     *p_orig;
+	gdouble    r_mul, g_mul, b_mul, a_mul;
+	gint       x = 0;
+	gint	   y = 0;
+
+	TransParam *tp = (TransParam *) trans_param;
+
+	if (!global_splash) {
+		g_free (tp);
+		return FALSE;
+	}
+	
+	if (tp->si->trans_count++ == TRANS_TIMEOUT_PERIOD) {
+		gtk_widget_queue_draw_area (GTK_WIDGET (tp->sw),
+					    tp->si->position.x, tp->si->position.y,
+					    tp->si->position.width, tp->si->position.height);
+		g_free (tp);
+		return FALSE;
+	}
+
+	a_mul = (gdouble) tp->si->trans_count / TRANS_TIMEOUT_PERIOD;
+	r_mul = 1;
+	g_mul = 1;
+	b_mul = 1;
+
+	for (y = 0; y < tp->height; y++) {
+		for (x = 0; x < tp->width; x++) {
+			p_trans = tp->pixels_trans + y * tp->rowstride_trans + x * tp->n_channels;
+			p_orig  = tp->pixels_orig  + y * tp->rowstride_orig  + x * tp->n_channels;
+
+			/* we can add more effects here apart from alpha fading */
+			p_trans[0] = r_mul * p_orig[0];
+			p_trans[1] = g_mul * p_orig[1];
+			p_trans[2] = b_mul * p_orig[2];
+			p_trans[3] = a_mul * p_orig[3];
+		}
+	}
+
+	gtk_widget_queue_draw_area (GTK_WIDGET (tp->sw),
+				    tp->si->position.x, tp->si->position.y,
+				    tp->si->position.width, tp->si->position.height);
+	return TRUE;
+}
+
 void
 splash_start (void)
 {
